# Libraries.R
# Wageningen Environmental Research
#________________________________________________
#

# initial part of procedure
#------------------------------------------------

# general settings
CRAN_mirror <- "0-Cloud [https]"
dir_pkg <- "./Tools/R/Libraries"

# set packages to load
pkg_load <- c("fs", "stringr", "dplyr", "readr", "doParallel", "doSNOW", "controlR", "KNMItools", "SWAPtools", "WWLanalyse", "WWLregionaal")

# set repositry
options(repos = c(getOption("repos")["CRAN"], "https://waterwijzerlandbouw.wur.nl/repo"))
online <- TRUE

# create local directory (optional)
if (!is.null(dir_pkg)) {
   dir.create(path = dir_pkg, recursive = TRUE, showWarnings =  FALSE)
   dir_pkg <- normalizePath(dir_pkg)
   .libPaths(dir_pkg)
  }

# main part of procedure
#------------------------------------------------

message("\nLoad packages...")

# set CRAN mirror in case packages need to be installed
if (online) {
   CRAN_mirrors <- getCRANmirrors()
   rec <- match(CRAN_mirror,CRAN_mirrors$Name)
   if (all(is.na(rec))) stop(paste("\nUnable to select CRANmirror: ", CRAN_mirror, sep=""))
   chooseCRANmirror(ind = rec[!is.na(rec)][1])
  }

# loop over packages
for (s_pkg in pkg_load) {
   if (suppressWarnings(suppressPackageStartupMessages(require(package = s_pkg, lib.loc = dir_pkg, character.only = TRUE)))) {
     message(paste0("- package '", s_pkg, "' loaded"))
    } else {
     if (!online) {
       stop(paste0("failed to load package: ", s_pkg))
      } else {
       suppressWarnings(suppressPackageStartupMessages(install.packages(pkg = s_pkg, dependencies = TRUE, lib = dir_pkg, quiet = TRUE)))
       if (suppressWarnings(suppressPackageStartupMessages(require(package = s_pkg, lib.loc = dir_pkg, character.only = TRUE)))) {
         message(paste0("- package '", s_pkg, "' installed"))
        } else {
         stop(paste0("failed to install package: ", s_pkg))
        }
      }
    }
  }
  
# check if there are updates available
if (online) {
   if (!all(!pkg_load %in% old.packages()[,"Package"])) {
     suppressWarnings(suppressPackageStartupMessages(update.packages(lib.loc = dir_pkg, ask = FALSE)))
    }
  }

