# WWL-regionaal
# Wageningen Environmental Research
# waterwijzerlandbouw@wur.nl
#------------------------------------------------

# set start of program
TIMPRGSTART <- Sys.time()

# ---- arguments ----

# set modus
test <- FALSE

# load arguments
if (!test) {
  command_args <- commandArgs(trailingOnly = TRUE)
  if (length(command_args) != 1) {
    stop("wrong usage of script, arguments should be:\n- ctrl")
  }
} else {
  command_args <- NULL
  command_args[1] <- "control_regionaal.inp"
  setwd("d:/Regionaal")
}

# set arguments
ctrl <- command_args[1]

# ---- load libraries ----

source("Tools/R/Libraries.R")

sign_open <- "\\{\\{"
sign_close <- "\\}\\}"

message("\nProgam R-PROG started...\n")

# ---- pre-process ----

# set directories
dir_run <- get_dir(file = ctrl, item = "DIRRUN", create = FALSE)

# set SWAP program and template
command <- get_record(file = ctrl, item = "PRGSWP")
tmplt_swp <- get_record(file = ctrl, item = "FILSWP")
tmplt_dra <- get_record(file = ctrl, item = "FILDRA", item_exists = FALSE)
dir_met <- get_record(file = ctrl, item = "DIRMET")
dir_ini <- get_record(file = ctrl, item = "DIRINI", item_exists = FALSE)
dir_crp <- get_record(file = ctrl, item = "DIRCRP")

# set type of meteodata
opt_meteo <- get_record(file = ctrl, item = "METEO", opt = c("STATION", "RADAR"), item_exists = FALSE)
if (is.null(opt_meteo)) opt_meteo <- "STATION"

# set database
file_sql <- get_record(file = ctrl, item = "FILSQL")

# set run_id
run_id <- get_record(file = ctrl, item = "RUNID")
if (run_id == "ALL") {
  run_id <- get_run_id_SQL(file_sql = file_sql)
} else {
  run_id <- string_sequence(string = run_id)
}

# processes
pre_process <- TRUE
record <- get_record(file = ctrl, item = "CREATE", opt = c("Yes", "No"), item_exists = FALSE)
if (!is.null(record)) pre_process <- ifelse(record == "Yes", TRUE, FALSE)
main_process <- TRUE
record <- get_record(file = ctrl, item = "RUN", opt = c("Yes", "No"), item_exists = FALSE)
if (!is.null(record)) main_process <- ifelse(record == "Yes", TRUE, FALSE)

# ---- main-process ----

# extract key_id from run-directory
key_id <- get_variable(text = dir_run)

# loop over run_id
pb <- progress_bar$new(total = length(run_id), format = "simulate SWAP :percent (run: :what)", clear = TRUE)
for (s_run_id in run_id) {
  
  # set progress
  pb$tick(tokens = list(what = s_run_id))
  
  # extract run_info
  run_info <- get_run_info_SQL(file_sql = file_sql, run_id = s_run_id)
  
  # set run-directory
  dir_tmp <- dir_run
  if (!is.null(key_id)) {
    for (s_key_id in key_id) {
      dir_tmp <- str_replace(string = dir_tmp, pattern = str_c(sign_open, s_key_id, sign_close), replacement = as.character(run_info[[s_key_id]]))
    }
  }
  
  # set swp-file
  file_swp <- str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), "/swap.swp")
  
  # --- pre-process SWAP ---
  
  if (pre_process) {
    
    # create meteorological data (optional)
    file_met <- str_c(dir_met, "/", run_info$meteo_id, ".met")
    if (opt_meteo == "RADAR") {
      file_met <- str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), "/temp/", run_info$meteo_id, "_", run_info$rain_id, ".met")
      create_meteorology(file_met = file_met, file_sql = file_sql, run_id = s_run_id, dir_met = dir_met)
    }
    
    # create SWAP input
    create_SWAP(file_swp = file_swp, file_sql = file_sql, run_id = s_run_id, tmplt_swp = tmplt_swp, dir_met = path_dir(file_met), dir_atm = dir_met, dir_ini = dir_ini, dir_crp = dir_crp, tmplt_dra = tmplt_dra, quiet = TRUE)
  }

  # --- main-process SWAP ---
  
  if (main_process) {
    
    # execute SWAP program
    run_SWAP(file_swp = file_swp, command = command, quiet = TRUE)
  
    # post-process SWAP
    zip_SWAP(file_swp = file_swp, glob = "*.swp|*.crp|*.csv|*.end")
    path <- str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), "/swap.zip")
    new_path <- str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), ".zip")
    if (file_exists(file = path)) {
      file_move(path = path, new_path = new_path)
      dir_delete(path = str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0")))
    }
  }
}

# return part of procedure
#------------------------------------------------
TIMPRGEND <- Sys.time()
TIMPRGCALC <- as.numeric(difftime(time1 = TIMPRGEND, time2 = TIMPRGSTART, units = "secs"))
message(paste0("\nProgam R-PROG successfully ended in ", floor(TIMPRGCALC / 60), " minutes and ", ceiling(TIMPRGCALC %% 60), " seconds"))
q(save = "no")
