# WWL-regionaal
# Wageningen Environmental Research
# waterwijzerlandbouw@wur.nl
#------------------------------------------------

# set start of program
TIMPRGSTART <- Sys.time()

# ---- arguments ----

# set modus
test <- FALSE

# load arguments
if (!test) {
  command_args <- commandArgs(trailingOnly = TRUE)
  if (length(command_args) != 1) {
    stop("wrong usage of script, arguments should be:\n- ctrl")
  }
} else {
  command_args <- NULL
  command_args[1] <- "control_regionaal.inp"
  setwd("d:/Regionaal")
}

# set arguments
ctrl <- command_args[1]

# ---- load libraries ----

source("Tools/R/Libraries_multicore.R")

sign_open <- "\\{\\{"
sign_close <- "\\}\\}"

message("\nProgam R-PROG started...\n")

# ---- pre-process ----

# set directories
dir_run <- get_dir(file = ctrl, item = "DIRRUN", create = FALSE)

# set SWAP program and template
command <- get_record(file = ctrl, item = "PRGSWP")
tmplt_swp <- get_record(file = ctrl, item = "FILSWP")
tmplt_dra <- get_record(file = ctrl, item = "FILDRA", item_exists = FALSE)
dir_met <- get_record(file = ctrl, item = "DIRMET")
dir_ini <- get_record(file = ctrl, item = "DIRINI", item_exists = FALSE)
dir_crp <- get_record(file = ctrl, item = "DIRCRP")

# set type of meteodata
opt_meteo <- get_record(file = ctrl, item = "METEO", opt = c("STATION", "RADAR"), item_exists = FALSE)
if (is.null(opt_meteo)) opt_meteo <- "STATION"

# set database
file_sql <- get_record(file = ctrl, item = "FILSQL")

# set run_id
run_id <- get_record(file = ctrl, item = "RUNID")
if (run_id == "ALL") {
  run_id <- get_run_id_SQL(file_sql = file_sql)
} else {
  run_id <- string_sequence(string = run_id)
}

# processes
pre_process <- TRUE
record <- get_record(file = ctrl, item = "CREATE", opt = c("Yes", "No"), item_exists = FALSE)
if (!is.null(record)) pre_process <- ifelse(record == "Yes", TRUE, FALSE)
main_process <- TRUE
record <- get_record(file = ctrl, item = "RUN", opt = c("Yes", "No"), item_exists = FALSE)
if (!is.null(record)) main_process <- ifelse(record == "Yes", TRUE, FALSE)

# set number of nodes
num_cpu <- get_record(file = ctrl, item = "CPU")
max_attempt <- get_record(file = ctrl, item = "MAX_ATTEMPT")


# ---- main-process ----

# create cluster
message("create cluster...")
cluster <- makeCluster(num_cpu, type = "SOCK")
registerDoSNOW(cl = cluster)
message(str_c("- cluster running with ", getDoParWorkers(), " worker(s)\n"))

# set template(s)
files_swp <- NULL
files_met <- NULL
key_id <- get_variable(text = dir_run)
for (s_run_id in run_id) {

  # extract run_info
  run_info <- get_run_info_SQL(file_sql = file_sql, run_id = s_run_id)

  # set run-directory
  dir_tmp <- dir_run
  if (!is.null(key_id)) {
    for (s_key_id in key_id) {
      dir_tmp <- str_replace(string = dir_tmp, pattern = str_c(sign_open, s_key_id, sign_close), replacement = as.character(run_info[[s_key_id]]))
    }
  }

  # set swp-file
  files_swp <- c(files_swp, str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), "/swap.swp"))

  # set met-file
  if (opt_meteo == "STATION") files_met <- c(files_met, str_c(dir_met, "/", run_info$meteo_id, ".met"))
  if (opt_meteo == "RADAR") files_met <- c(files_met, str_c(dir_tmp, "/run_", formatC(x = s_run_id, format = "d", width = 9, flag = "0"), "/temp/", run_info$meteo_id, "_", run_info$rain_id, ".met"))
}

# -- pre-process --

if (pre_process) {

  if (opt_meteo == "RADAR") {

    file_err <- "./failed_create.err"
    if (controlR::file_exists(file = file_err, quiet = TRUE)) file_delete(path = file_err)

    message("create input rainfall...", appendLF = FALSE)

    tmp_id <- run_id
    file_tmp <- files_met
    attempt <- 1
    done <- FALSE
    while (!done) {

      return <- foreach (s_run_id = tmp_id, s_file_met = file_tmp, .errorhandling = "remove", .packages = ("WWLregionaal")) %dopar% create_meteorology(file_met = s_file_met, file_sql = file_sql, run_id = s_run_id, dir_met = dir_met)

      # check attempt
      succes_id <- as.numeric(str_sub(string = path_file(str_replace_all(path_dir(dir_ls(path = dir_run, recurse = TRUE, type = "file", glob = "*.met")), pattern = "/temp", replacement = "")), start = 5, end = 13))
      failed_id <- run_id[!run_id %in% succes_id]
      if (length(failed_id) == 0 | attempt == max_attempt) {
        done <- TRUE
      } else {
        attempt <- attempt + 1
        tmp_id <- failed_id
        file_tmp <- files_met[match(x = failed_id, table = run_id)]
      }
    }
    message(" done")

    # message to screen
    if (length(failed_id) != 0) {
      message(str_c("failed to created rainfall ", length(failed_id), " runs (after ", max_attempt, " attempts)"))
      write_lines(x = sequence_string(failed_id), file = file_err)
    }
  }

  message("create input...", appendLF = FALSE)

  tmp_id <- run_id
  file_tmp <- files_swp
  file_met <- files_met
  attempt <- 1
  done <- FALSE
  while (!done) {

    return <- foreach (s_run_id = tmp_id, s_file_swp = file_tmp, s_dir_met = path_dir(file_met), .errorhandling = "remove", .packages = ("SWAPtools")) %dopar% create_SWAP(file_swp = s_file_swp, file_sql = file_sql, run_id = s_run_id, tmplt_swp = tmplt_swp, dir_met = s_dir_met, dir_atm = dir_met, dir_ini = dir_ini, dir_crp = dir_crp, tmplt_dra = tmplt_dra, quiet = TRUE)

    # check attempt
    succes_id <- as.numeric(str_sub(string = path_file(path_dir(dir_ls(path = dir_run, recurse = TRUE, type = "file", glob = "*swap.swp"))), start = 5, end = 13))
    failed_id <- run_id[!run_id %in% succes_id]
    if (length(failed_id) == 0 | attempt == max_attempt) {
      done <- TRUE
    } else {
      attempt <- attempt + 1
      tmp_id <- failed_id
      file_tmp <- files_swp[match(x = failed_id, table = run_id)]
      file_met <- files_met[match(x = failed_id, table = run_id)]
    }
  }
  message(" done")

  # message to screen
  if (length(failed_id) != 0) {
    message(str_c("failed to created ", length(failed_id), " runs (after ", max_attempt, " attempts)"))
    write_lines(x = sequence_string(failed_id), file = file_err)
  }
}

# -- main-process --

if (main_process) {

  file_err <- "./failed_execute.err"
  if (controlR::file_exists(file = file_err, quiet = TRUE)) file_delete(path = file_err)

  # run SWAP
  message("run SWAP...", appendLF = FALSE)
  return <- foreach (s_file_swp = files_swp, .errorhandling = "remove", .packages = ("SWAPtools")) %dopar% run_SWAP(file_swp = s_file_swp, command = command, quiet = TRUE)
  message(" done")

  # message to screen
  succes_id <- as.numeric(str_sub(string = path_file(path_dir(dir_ls(path = dir_run, recurse = TRUE, type = "file", glob = "*swap.ok"))), start = 5, end = 13))
  failed_id <- run_id[!run_id %in% succes_id]
  if (length(failed_id) != 0) {
    message(str_c("failed to execute ", length(failed_id), " runs"))
    write_lines(x = sequence_string(failed_id), file = file_err)
  }

  # zip results
  message("zip results...", appendLF = FALSE)
  return <- foreach (s_file_swp = files_swp, .errorhandling = "remove", .packages = ("SWAPtools")) %dopar% zip_SWAP(file_swp = s_file_swp, glob = "*.swp|*.crp|*.csv|*.end")
  message(" done")
}

# stop cluster
stopCluster(cl = cluster)

if (main_process) {

  # rename files
  succes_id <- as.numeric(str_sub(string = path_file(path_dir(dir_ls(path = dir_run, recurse = TRUE, type = "file", glob = "*swap.zip"))), start = 5, end = 13))
  path <- str_c(dir_tmp, "/run_", formatC(x = succes_id, format = "d", width = 9, flag = "0"), "/swap.zip")
  new_path <- str_c(dir_tmp, "/run_", formatC(x = succes_id, format = "d", width = 9, flag = "0"), ".zip")
  if (controlR::file_exists(file = path, quiet = TRUE)) {
    file_move(path = path, new_path = new_path)
    dir_delete(path = str_c(dir_tmp, "/run_", formatC(x = run_id, format = "d", width = 9, flag = "0")))
  }
}

# return part of procedure
#------------------------------------------------
TIMPRGEND <- Sys.time()
TIMPRGCALC <- as.numeric(difftime(time1 = TIMPRGEND, time2 = TIMPRGSTART, units = "secs"))
message(paste0("\nProgam R-PROG successfully ended in ", floor(TIMPRGCALC / 60), " minutes and ", ceiling(TIMPRGCALC %% 60), " seconds"))
q(save = "no")
